@@ -1,5 +1,5 @@
 library(metR)
-library(vdiffr)
+# library(vdiffr)
 
 context("contour_fill")
 
