@@ -1,6 +1,6 @@
 
 context("tex_contour")
-v <- data.table::melt(volcano)
+v <- reshape2::melt(volcano)
 g <- ggplot(v, aes(Var1, Var2, z = value)) +
     geom_contour()
 
